@@ -4,15 +4,17 @@
 #'   screen and captured.
 #' @return list containing four functions: \code{get_new}, \code{pause},
 #'  \code{unpause}, \code{close}.
+#' @keywords internal
 watchout <- function(debug = FALSE) {
   output <- vector("character")
   prev   <- vector("character")
 
-  con <- textConnection("output", "wr", local=TRUE)
+  con <- textConnection("output", "wr", local = TRUE)
   sink(con, split = debug)
 
   list(
-    get_new = function(plot = FALSE, incomplete_plots = FALSE) {
+    get_new = function(plot = FALSE, incomplete_plots = FALSE,
+                       text_callback = identity, graphics_callback = identity) {
       incomplete <- isIncomplete(con)
       if (incomplete) cat("\n")
 
@@ -20,6 +22,7 @@ watchout <- function(debug = FALSE) {
 
       if (plot) {
         out$graphics <- plot_snapshot(incomplete_plots)
+        if (!is.null(out$graphics)) graphics_callback(out$graphics)
       }
 
       if (length(output) != length(prev)) {
@@ -28,6 +31,8 @@ watchout <- function(debug = FALSE) {
 
         out$text <- str_c(new, collapse = "\n")
         if (!incomplete) out$text <- str_c(out$text, "\n")
+
+        text_callback(out$text)
       }
 
       unname(out)
